﻿using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.Extensions.Logging;
using ModrinkyWebApp.Models;
using System.ComponentModel.DataAnnotations;
using System.Threading.Tasks;

namespace ModrinkyWebApp.Areas.Identity.Pages.Account.Manage
{
    public class ChangePasswordModel : PageModel
    {
        private readonly UserManager<Uzivatel> _userManager;
        private readonly SignInManager<Uzivatel> _signInManager;
        private readonly ILogger<ChangePasswordModel> _logger;

        public ChangePasswordModel(
            UserManager<Uzivatel> userManager,
            SignInManager<Uzivatel> signInManager,
            ILogger<ChangePasswordModel> logger)
        {
            _userManager = userManager;
            _signInManager = signInManager;
            _logger = logger;
        }

        [BindProperty]
        public InputModel Input { get; set; }

        [TempData]
        public string StatusMessage { get; set; }

        public class InputModel
        {
            [Required(ErrorMessage = "Vypnění starého hesla je vyžadováno")]
            [DataType(DataType.Password, ErrorMessage = "Špatné heslo")]
            [Display(Name = "Současné heslo")]
            public string OldPassword { get; set; }

            [Required(ErrorMessage = "Vypnění nového hesla je vyžadováno")]
            [StringLength(100, ErrorMessage = "Heslo může být dlouhé minimálně {2} a maximálně {1} znaků.", MinimumLength = 6)]
            [DataType(DataType.Password, ErrorMessage = "Špatný formát hesla.")]
            [Display(Name = "Nové heslo")]
            public string NewPassword { get; set; }

            [DataType(DataType.Password, ErrorMessage = "Špatný formát hesla.")]
            [Display(Name = "Potvrzení nového hesla")]
            [Compare("NewPassword", ErrorMessage = "Hesla se neshodují.")]
            public string ConfirmPassword { get; set; }
        }

        public async Task<IActionResult> OnGetAsync()
        {
            var user = await _userManager.GetUserAsync(User);
            if (user == null)
            {
                return NotFound($"Unable to load user with ID '{_userManager.GetUserId(User)}'.");
            }

            var hasPassword = await _userManager.HasPasswordAsync(user);
            if (!hasPassword)
            {
                return RedirectToPage("./SetPassword");
            }

            return Page();
        }

        public async Task<IActionResult> OnPostAsync()
        {
            if (!ModelState.IsValid)
            {
                return Page();
            }

            var user = await _userManager.GetUserAsync(User);
            if (user == null)
            {
                return NotFound($"Unable to load user with ID '{_userManager.GetUserId(User)}'.");
            }

            var changePasswordResult = await _userManager.ChangePasswordAsync(user, Input.OldPassword, Input.NewPassword);
            if (!changePasswordResult.Succeeded)
            {
                foreach (var error in changePasswordResult.Errors)
                {
                    ModelState.AddModelError(string.Empty, "Špatné heslo");
                }
                return Page();
            }

            await _signInManager.RefreshSignInAsync(user);
            TempData["Zprava"] = "Heslo změněno";

            return RedirectToPage();
        }
    }
}